"""Error logging for jiratools."""
from jiratools import add_comment, format_as_code_block
from jiratools.formatting import format_autoupdate_jira_msg, format_as_jira_table


def add_jira_error_comment(jira_id, error_msg, **format_kwargs):
    """
    Add a comment to a JIRA with a formatted error message.

    Args:
        jira_id (str): the Issue ID of the JIRA to be updated
        error_msg (str): the raw error message to include in the comment
        **format_kwargs: formatting keyword args
            to be passed to jiratools.formatting.format_jira_msg

    Returns:
        str: the Issue ID of the JIRA which received the comment

    """
    add_comment(
        jira_id,
        format_autoupdate_jira_msg(format_as_code_block(error_msg), **format_kwargs),
    )
    return jira_id


def add_jira_comment_with_table(
    jira_id, data_headers, data_array, msg_prefix="", **format_kwargs
):
    """
    Add a comment to a JIRA with a formatted data table.

    Args:
        jira_id (str): the Issue ID of the JIRA to be updated
        data_headers (list of str): a list of header column names
        data_array (list of list of str,int): An array of lists of strings,
            representing table rows. e.g.::

            [["a", "b", "c"], ["d", "e", "f"]]

        **format_kwargs: formatting keyword args
            to be passed to jiratools.formatting.format_jira_msg

    Returns:
        str: the Issue ID of the JIRA which received the comment

    """
    message_with_table = "{}{}".format(
        msg_prefix, format_as_jira_table(data_headers, data_array)
    )
    add_comment(
        jira_id, format_autoupdate_jira_msg(message_with_table, **format_kwargs)
    )
    return jira_id


def update_jira_for_errors(jiras, *errors, **format_kwargs):
    """
    Auto-Update JIRAs if errors are found that match the jira list.

    Args:
        jiras (list, obj): an iterable of objects
            that each contain two required attributes::

            - jira_id (str): the Issue Id of the JIRA to be updated
            - error_message (str): an error message substring which, if found,
                will trigger and update of the JIRA with the actual error message.

        *errors (str): an error message to be checked against the ``jiras`` for a match
        **format_kwargs: formatting keyword args
            to be passed to jiratools.formatting.format_jira_msg

    Returns:
        list(str): A list of JIRA Issue IDs that were updated.

    """
    jiras_commented = []
    for jira in jiras:
        for error in errors:
            if jira.error_message and jira.error_message in error:
                jiras_commented.append(
                    add_jira_error_comment(jira.jira_id, error, **format_kwargs)
                )
    return jiras_commented



"""Formatting helpers for jiratools."""
import getpass
import os


def _build_source():
    return os.environ.get("BUILD_URL", "Manual run by {}".format(getpass.getuser()))


def format_autoupdate_jira_msg(message_body, header_body=None):
    """
    Format a JIRA message with useful headers.

    An "Automated JIRA Update" title will be added,
    as well as either a URL link if a ``BUILD_URL`` env variable is present,
    or a note indicating a manual run with user id otherwise.

    Args:
        message_body (str): the body of the message
        header_body (str, optional): a header to be added with ``h2`` tag

    Returns:
        str: a formatted message with headers

    """
    message = "h2. {}".format(header_body) if header_body else ""
    message += "\n\nAutomated JIRA Update:\n\n{}\n\n{}".format(
        _build_source(), message_body
    )
    return message


def format_as_jira_table(headers, data_array):
    """
    Build a JIRA table given headers and row data.

    Args:
        headers (list of str): a list of header column names
        data_array (list of list of str,int): An array of lists of strings,
            representing table rows. e.g.::

            [["a", "b", "c"], ["d", "e", "f"]]

    Returns:
        str: a formatted JIRA table

    """
    headers = ["||{}||".format("||".join(headers))]
    return "\n".join(
        headers + ["|{}|".format("|".join(map(str, d))) for d in data_array]
    )



# -*- coding: utf-8 -*-
from distutils.core import setup

packages = \
['jiratools']

package_data = \
{'': ['*']}

install_requires = \
['jira>=2.0,<3.0']

entry_points = \
{'console_scripts': ['jira-add-comment = jiratools:_cli_add_comment',
                     'jira-example-config = jiratools:_example_config_install',
                     'jira-link-issues = jiratools:cli_jira_link',
                     'jira-make-linked-issue = '
                     'jiratools:_create_test_jira_from',
                     'jira-search-issues = jiratools:_cli_search',
                     'jira-update-assignee = jiratools:_change_jira_assignee']}

setup_kwargs = {
    'name': 'jiratools',
    'version': '2.0.0',
    'description': 'Simple helpers to interface to JIRA from an API or command line.',
    'long_description': None,
    'author': 'Brad Brown',
    'author_email': 'brad@bradsbrown.com',
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)
